@@ -14,6 +14,7 @@ from functools import reduce
 import urwid
 
 from vit import version
+from vit.exception import VitException
 from vit.formatter_base import FormatterBase
 from vit import event
 from vit.loader import Loader
@@ -96,23 +97,28 @@ class Application():
     def set_active_context(self):
         self.context = self.task_config.get_active_context()
 
+    def load_contexts(self):
+        self.contexts = self.task_config.get_contexts()
+
     def bootstrap(self, load_early_config=True):
         self.loader = Loader()
         if load_early_config:
             self.load_early_config()
-        self.contexts = self.task_config.get_contexts()
+        self.load_contexts()
         self.set_active_context()
         self.event = event.Emitter()
         self.setup_config()
         self.search_term_active = ''
+        self.search_direction_reverse = False
         self.action_registry = ActionRegistry()
         self.actions = Actions(self.action_registry)
         self.actions.register()
         self.keybinding_parser = KeybindingParser(self.loader, self.config, self.action_registry)
+        self.command = Command(self.config)
+        self.get_available_task_columns()
         self.setup_keybindings()
         self.action_manager = ActionManagerRegistry(self.action_registry, self.key_cache.keybindings, event=self.event)
         self.register_managed_actions()
-        self.command = Command(self.config)
         self.markers = Markers(self.config, self.task_config)
         self.theme = self.init_theme()
         self.theme_alt_backgrounds = self.get_theme_alt_backgrounds()
@@ -167,15 +173,57 @@ class Application():
         self.action_manager_registrar.register('TASK_EDIT', self.task_action_edit)
         self.action_manager_registrar.register('TASK_SHOW', self.task_action_show)
 
+    def default_keybinding_replacements(self):
+        import json
+        from datetime import datetime
+        task_replacement_match = re.compile("^TASK_(\w+)$")
+        def _task_attribute_match(variable):
+            matches = re.match(task_replacement_match, variable)
+            if matches:
+                attribute = matches.group(1).lower()
+                if attribute in self.available_columns:
+                    return [attribute]
+        def _task_attribute_replace(task, attribute):
+            if task and task[attribute]:
+                if type(task[attribute]) in ['set', 'tuple', 'dict', 'list']:
+                    try:
+                        json.dumps(task[attribute])
+                    except Exception as e:
+                        raise RuntimeError('Error parsing task attribute %s as JSON: %s' % (task[attribute], e))
+                elif isinstance(task[attribute], datetime):
+                    return task[attribute].strftime(self.formatter.report)
+                else:
+                    return str(task[attribute])
+            return ''
+        replacements = [
+            {
+                'match_callback': _task_attribute_match,
+                'replacement_callback': _task_attribute_replace,
+            },
+        ]
+        return replacements
+
+    def add_user_keybinding_replacements(self, replacements):
+        klass = self.loader.load_user_class('keybinding', 'keybinding', 'Keybinding')
+        if klass:
+            keybinding_custom = klass()
+            replacements.extend(keybinding_custom.replacements())
+        return replacements
+
+    def wrap_replacements_callbacks(self, replacements):
+        def build_wrapper(callback):
+            def wrapper(*args):
+                _, task = self.get_focused_task()
+                return callback(task, *args)
+            return wrapper
+        for i, replacement in enumerate(replacements):
+            replacements[i]['replacement_callback'] = build_wrapper(replacement['replacement_callback'])
+        return replacements
+
     def setup_keybindings(self):
         self.keybinding_parser.load_default_keybindings()
         bindings = self.config.items('keybinding')
-        def _task_uuid():
-            uuid, _ = self.get_focused_task()
-            return uuid
-        replacements = {
-            'TASK_UUID': _task_uuid,
-        }
+        replacements = self.wrap_replacements_callbacks(self.add_user_keybinding_replacements(self.default_keybinding_replacements()))
         keybindings = self.keybinding_parser.add_keybindings(bindings=bindings, replacements=replacements)
         self.key_cache = KeyCache(keybindings)
         self.key_cache.build_multi_key_cache()
@@ -228,8 +276,8 @@ class Application():
 
     def prepare_keybinding_keypresses(self, keypresses):
         def reducer(accum, key):
-            if isfunction(key):
-                accum += list(key())
+            if type(key) is tuple:
+                accum += list(key[0](*key[1]))
             else:
                 accum.append(key)
             return accum
@@ -296,6 +344,9 @@ class Application():
             elif op == 'context':
                 # TODO: Validation if more than one arg passed.
                 context = args[0] if len(args) > 0 else 'none'
+                if context != 'none':
+                    # In case a new context was added between bootstraps.
+                    self.load_contexts()
                 if self.execute_command(['task', 'context', context], wait=self.wait):
                     self.activate_message_bar('Context switched to: %s' % context)
                 else:
@@ -323,6 +374,7 @@ class Application():
                         self.activate_message_bar('Task %s tags updated' % self.model.task_id(task['uuid']))
                 elif op in ('search-forward', 'search-reverse'):
                     self.search_set_term(data['text'])
+                    self.search_set_direction(op)
                     self.search(reverse=(op == 'search-reverse'))
         self.widget.focus_position = 'body'
         if 'uuid' in metadata:
@@ -396,6 +448,8 @@ class Application():
                 self.update_report(command)
                 if 'uuid' in metadata:
                     metadata.pop('uuid')
+            elif command in self.task_config.disallowed_reports:
+                self.activate_message_bar("Report '%s' is non-standard, use ':!w task %s'" % (command, command), 'error')
             else:
                 # Matches 's/foo/bar/' and s%/foo/bar/, allowing for separators
                 # to be any non-word character.
@@ -415,6 +469,9 @@ class Application():
     def search_set_term(self, text):
         self.search_term_active = text
 
+    def search_set_direction(self, op):
+        self.search_direction_reverse = op == 'search-reverse'
+
     def search(self, reverse=False):
         if not self.search_term_active:
             return
@@ -505,7 +562,8 @@ class Application():
         self.autocomplete = AutoComplete(self.config, extra_filters={'report': self.reports.keys(), 'help': self.help.autocomplete_entries(), 'context': context_list})
 
     def init_command_bar(self):
-        self.command_bar = CommandBar(autocomplete=self.autocomplete, event=self.event)
+        abort_backspace = self.config.get('vit', 'abort_backspace')
+        self.command_bar = CommandBar(autocomplete=self.autocomplete, abort_backspace=abort_backspace, event=self.event)
 
     def build_frame(self):
         self.status_report = urwid.AttrMap(urwid.Text('Welcome to VIT'), 'status')
@@ -614,10 +672,10 @@ class Application():
         self.activate_command_bar('search-reverse', '?', {'history': 'search'})
 
     def activate_command_bar_search_next(self):
-        self.search()
+        self.search(reverse=self.search_direction_reverse)
 
     def activate_command_bar_search_previous(self):
-        self.search(reverse=True)
+        self.search(reverse=not self.search_direction_reverse)
 
     def activate_command_bar_task_context(self):
         self.activate_command_bar('context', 'Context: ')
@@ -703,13 +761,11 @@ class Application():
     def task_action_priority(self):
         uuid, _ = self.get_focused_task()
         if uuid:
-            choices = {
-                'h': 'H',
-                'm': 'M',
-                'l': 'L',
-                'n': '',
-            }
-            self.activate_command_bar('priority', 'Priority (h/m/l/n): ', {'uuid': uuid, 'choices': choices})
+            choices = {}
+            for choice in self.task_config.priority_values:
+                key = choice.lower() or 'n'
+                choices[key] = choice
+            self.activate_command_bar('priority', 'Priority (%s): ' % '/'.join(choices), {'uuid': uuid, 'choices': choices})
 
     def task_action_project(self):
         uuid, _ = self.get_focused_task()
@@ -738,6 +794,13 @@ class Application():
             self.execute_command(['task', uuid, 'info'], update_report=False)
             self.task_list.focus_by_task_uuid(uuid)
 
+    def get_available_task_columns(self):
+        returncode, stdout, stderr = self.command.run(['task', '_columns'], capture_output=True)
+        if returncode == 0:
+            self.available_columns = stdout.split()
+        else:
+            raise RuntimeError("Error retrieving available task columns: %s" % stderr)
+
     def refresh_blocking_task_uuids(self):
         returncode, stdout, stderr = self.command.run(['task', 'uuids', '+BLOCKING'], capture_output=True)
         if returncode == 0:
@@ -847,7 +910,11 @@ class Application():
         self.refresh_blocking_task_uuids()
         self.formatter.recalculate_due_datetimes()
         context_filters = self.contexts[self.context]['filter'] if self.context else []
-        self.model.update_report(self.report, context_filters=context_filters, extra_filters=self.extra_filters)
+        try:
+            self.model.update_report(self.report, context_filters=context_filters, extra_filters=self.extra_filters)
+        except VitException as err:
+            self.activate_message_bar(str(err), 'error')
+            return
         self.update_task_table()
         self.update_status_report()
         self.update_status_context()
