@@ -89,25 +89,25 @@ class TaskTable(object):
         self.indent_subprojects = self.subproject_indentable()
         self.project_cache = {}
         # TODO: This is for the project placeholders, feels sloppy.
-        self.project_formatter = ProjectFormatter('project', self.report, self.formatter)
+        self.project_formatter = ProjectFormatter('project', self.report, self.formatter, self.get_blocking_task_uuids())
         self.build_rows()
         self.clean_columns()
-        self.has_project_column = self.project_column_present()
-        self.resize_columns()
+        self.project_column_idx = self.get_project_column_idx()
         self.reconcile_column_width_for_label()
+        self.resize_columns()
         self.build_table()
         self.listbox.set_focus_position()
         self.update_focus()
 
     def update_header(self, size):
-        if self.has_project_column:
+        if self.project_column_idx is not None:
             self.update_project_column_header(size)
 
-    def project_column_present(self):
-        for _, column in enumerate(self.columns):
+    def get_project_column_idx(self):
+        for idx, column in enumerate(self.columns):
             if column['name'] == 'project':
-                return True
-        return False
+                return idx
+        return None
 
     def get_project_from_row(self, row):
         return row.task['project'] if isinstance(row, SelectableRow) else row.project
@@ -124,9 +124,7 @@ class TaskTable(object):
                     self.set_project_column_header()
 
     def set_project_column_header(self, parents=None):
-        column_index = self.task_config.get_column_index(self.report['name'], 'project')
-        if self.has_marker_column():
-            column_index += 1
+        column_index = self.project_column_idx
         (columns_widget, _) = self.header.original_widget.contents[column_index]
         (widget, _) = columns_widget.contents[0]
         label = self.project_label_for_parents(parents)
@@ -221,7 +219,7 @@ class TaskTable(object):
         kwargs = self.column_formatter_kwargs()
         for idx, column_formatter in enumerate(self.report['columns']):
             name, formatter_class = self.formatter.get(column_formatter)
-            self.add_column(name, self.report['labels'][idx], formatter_class(name, self.report, self.formatter, **kwargs))
+            self.add_column(name, self.report['labels'][idx], formatter_class(name, self.report, self.formatter, self.get_blocking_task_uuids(), **kwargs))
 
     def is_marker_column(self, column):
         return column == MARKER_COLUMN_NAME
@@ -315,16 +313,18 @@ class TaskTable(object):
                 to_adjust.append({'idx': idx, 'width': width})
         if total_width > cols:
             self.adjust_oversized_columns(total_width - cols, to_adjust)
+            if to_adjust:
+                # This is called recursively to account for cases when further
+                # reduction is necessary because one or more column's reductions
+                # were limited to REDUCE_COLUMN_WIDTH_LIMIT.
+                self.resize_columns()
 
     def adjust_oversized_columns(self, reduce_by, to_adjust):
         to_adjust = list(map(lambda c: c.update({'ratio': (c['width'] - REDUCE_COLUMN_WIDTH_LIMIT) / c['width']}) or c, to_adjust))
         ratio_total = reduce(lambda acc, c: acc + c['ratio'], to_adjust, 0)
         to_adjust = list(map(lambda c: c.update({'percentage': c['ratio'] / ratio_total}) or c, to_adjust))
         for c in to_adjust:
-            # This shouldn't technically be necessary, but there are edge cases
-            # when a single adjusted column is 1 column too wide for the
-            # display, which results in it not being displayed.
-            adjusted_width = c['width'] - (reduce_by + 1 if c['percentage'] == 1 else math.ceil(reduce_by * c['percentage']))
+            adjusted_width = c['width'] - math.ceil(reduce_by * c['percentage'])
             self.columns[c['idx']]['width'] = adjusted_width if adjusted_width > REDUCE_COLUMN_WIDTH_LIMIT else REDUCE_COLUMN_WIDTH_LIMIT
 
     def reconcile_column_width_for_label(self):
@@ -365,10 +365,11 @@ class TaskTable(object):
         self.header = urwid.AttrMap(list_header, 'list-header')
 
     def make_header_column(self, column, is_last, space_between=COLUMN_PADDING):
-        total_width = column['width'] + space_between
+        padding_width = 0 if is_last else space_between
+        total_width = column['width'] + padding_width
         column_content = urwid.AttrMap(urwid.Padding(urwid.Text(column['label'], align='left')), 'list-header-column')
         padding_content = self.make_padding(is_last and 'list-header-column' or 'list-header-column-separator')
-        columns = urwid.Columns([(column['width'], column_content), (space_between, padding_content)])
+        columns = urwid.Columns([(column['width'], column_content), (padding_width, padding_content)])
         return (total_width, columns)
 
     def make_padding(self, display_attr):
@@ -442,10 +443,6 @@ class SelectableRow(urwid.WidgetWrap):
             key = self.on_select(self, size, key)
         return key
 
-        # ... and update the displayed items.
-        for t, (w, _) in zip(contents, self._columns.contents):
-            w.set_text(t)
-
 class ProjectPlaceholderRow(urwid.WidgetWrap):
     """Wraps 'urwid.Columns' for a project placeholder row.
     """
